<?php

/*

GPXファイルの標高を優先するかどうかのオプションを作って、有効の場合には、KMLファイルに標高も含める。


*/
class GTOGM_MakeKML {
	
	
	//
	//  コンストラクタ
	//
	function __construct() {
		
		// メディアファイルをアップロードした時のフック
		add_filter("wp_handle_upload", array(&$this, "gtogm_handle_upload"));		
		
		// メディアの追加画面にGPXファイルアップロード時のみにフィールドを追加
		add_filter("attachment_fields_to_edit", array(&$this, "gtogm_attachment_fields_to_edit"), 1, 2);
		
		// 追加したフィールドのアタッチメントを登録する。
		add_filter("attachment_fields_to_save", array(&$this, 'gtogm_attachment_fields_to_save'), 1, 2);
		
		// KMLファイルのショートコードを投稿エディタに挿入
		add_filter("media_send_to_editor", array(&$this, "gtogm_media_send_to_editor"), 1, 3);
		
		// アップロードできる拡張子を追加する。
		add_filter('upload_mimes', array(&$this, 'add_mimes'));
	}
	
	//
	// メディアの追加でアップロードできる拡張子を増やす
	//
	function add_mimes($mimes) {
		$mime_types = array ('gpx' => 'application/gpx');
		return array_merge($mimes, $mime_types);
	}
	
	// 
	// メディア追加画面から「投稿に挿入」ボタン押下時の挿入HTMLのフック
	// 
	function gtogm_media_send_to_editor($html, $send_id, $attachment) {
		
		// ショートコードをエディタに挿入
		if (isset($attachment["insert_type"]) && isset($attachment["kml_shortcode"])) {
			if ($attachment["insert_type"] == "shortcode") {
				$html = $attachment["kml_shortcode"];
			}
		}
		
		return $html;
	}
	
	// 
	// アタッチメント保存のフック
	// 
	function gtogm_attachment_fields_to_save($post, $attachment) {
//		if ($attachment["kml_shortcode"] == "shortcode") {
//			$post["kml_shortcode"] = "shortcode";
//		} else {
//			$post["kml_shortcode"] = "link";
//		}
		
//		if (isset($attachment['kml_shortcode'])) {
//			$post["kml_shortcode"] = "shortcode";
//		}
		
		return $post;
		
	}
	
	// 
	// メディアの追加画面に独自のFormフィールド（アタッチメント）を追加するフック
	// 
	function gtogm_attachment_fields_to_edit($form_fields, $post) {
		
		
		$gpx_file = wp_get_attachment_url($post->ID);
//		if (preg_match("/\.([gG][pP][xX]|[kK][mM][lL])$/", $gpx_file) == 1) {
		if (preg_match("/\.([gG][pP][xX])$/", $gpx_file) == 1) {
			
			// 
			$kml_file = preg_replace("/\.[gG][pP][xX]$/", ".kml", $gpx_file);
			
			// KMLリンク挿入ボタン追加
			$form_fields["url"]["html"] .= '<button type="button" class="button urlkml" title="' . $kml_file . '">KML File URL</button>';
			
			// プラグイン情報管理インスタンス生成
			$gtogm_info = new GTOGM_Info();
			
			// [gmap id="230" width="640" height="480" clat="36.562393" clon="137.630271" zoom="auto" type="G_NORMAL_MAP" control="2" glink="1"]
			$kml_id = $this->get_kml_postid($gpx_file);
			
			
			$id = "id=&quot;" . $kml_id . "&quot;";
			$width = "width=&quot;" . $gtogm_info->get_option("map_width") . "&quot;";
			$height = "height=&quot;" . $gtogm_info->get_option("map_height") . "&quot;";
			$meta = wp_get_attachment_metadata($kml_id);
			$clat = "clat=&quot;" . $meta["center_lat"] . "&quot;";
			$clon = "clon=&quot;" . $meta["center_lon"] . "&quot;";
			$zoom = "zoom=&quot;" . $gtogm_info->get_option("zoom_level") . "&quot;";
			$type = "type=&quot;" . $gtogm_info->get_option("map_type") . "&quot;";
			$control_value = 0;
			foreach ($gtogm_info->get_option("control") as $value) {
				$control_value += $value;
			}
			$control = "control=&quot;" . $control_value . "&quot;";
			$mtcontrol_value = 0;
			foreach ($gtogm_info->get_option("map_type_control") as $value) {
				$mtcontrol_value += $value;
			}
			$mtcontrol = "mtcontrol=&quot;" . $mtcontrol_value . "&quot;";
			$glink = "glink=&quot;" . $gtogm_info->get_option("open_google_maps") . "&quot;";
			$shortcode = "[gmap $id $width $height $clat $clon $zoom $type $control $mtcontrol $glink /]";
			
			// formにフィールドを追加するが、その際、フィールドのname属性が重要。 name="attachments[Post-ID][任意文字列]"
			$form_fields["kml_shortcode"] = array("label" => __("Google Maps Shortcode", "gtogm"), "input" => "html", "html" => "<input name=\"attachments[$post->ID][kml_shortcode]\" type=\"text\" value=\"$shortcode\" />", "helps" => "");
			$form_fields["insert_type"] = array("label" => __("Insert Type", "gtogm"), "input" => "html", "html" => "<input name=\"attachments[$post->ID][insert_type]\" type=\"radio\" value=\"shortcode\" checked />" . __("Google Maps Shortcode", "gtogm") . "&nbsp;&nbsp;&nbsp;&nbsp;<input name=\"attachments[$post->ID][insert_type]\" type=\"radio\" value=\"link\" />" . __("Link URL", "gtogm"), "helps" => "");
		}
		return $form_fields;
		
	}
	
	// 
	// メディアの追加画面にてファイルアップロード時のフック
	// 
	//function gtogm_handle_upload($file, $overrides = false, $time = null) {
	function gtogm_handle_upload($file) {
		
		global $post;
		
		// $file["file"]  $file["url"]  $file["type"]
		
		if (preg_match("/\.[gG][pP][xX]$/", $file["file"]) != 1) {
			return $file;
		}
		
		$gpx_filepath = $file["file"];
		$kml_filepath = preg_replace("/\.[gG][pP][xX]$/", ".kml", $gpx_filepath);
		
		$gpx_url = $file["url"];
		$kml_url = preg_replace("/\.[gG][pP][xX]$/", ".kml", $gpx_url);
		
		// プラグイン情報管理インスタンス生成
		$gtogm_info = new GTOGM_Info();
		
//		print("|".$gtogm_info->get_option("icon")."|");
		if ($gtogm_info->get_option("icon") == "") {
			// web上のファイル（アイコン）の存在チェック
			//     存在する場合の返り値  ：HTTP/1.1 200 OK
			//     存在しない場合の返り値：HTTP/1.1 404 Not Found
			$icon_url = $gtogm_info->get_option("icon_url");
		} else {
			$icon_url = $gtogm_info->get_option("icon");
		}
		$headers = get_headers($icon_url);
		if(preg_match("/ 200 OK$/", $headers[0])){
			// アイコンのサイズ取得
			list($width, $height, $type, $attr) = getimagesize($icon_url);
		}
		
		// KML生成
		$kml2 = new KMLFile($gpx_filepath, 
		                    $gtogm_info->get_option("line_color"), 
		                    $gtogm_info->get_option("line_weight"), 
		                    $gtogm_info->get_option("line_opacity"), 
		                    $gtogm_info->get_option("kashmir"),
							$icon_url,
							$width,
							$height,
							$gtogm_info->get_option("label_color")
		                    );
		
		// KMLファイルをメディアライブラリに登録（wp_postsテーブルにレコード(ポストデータ)追加）
		$post_id = 0;
		$attachment = array(
			'post_mime_type' => "application/kml",
			'guid' => $kml_url,
			'post_parent' => $post_id,
			'post_content' => '',
			'post_title' => basename($kml_filepath,".kml"),
			'post_name' => basename($kml_filepath,".kml")
		);
		$kml_post_id = wp_insert_attachment($attachment, $kml_filepath, $post_id);
		
				
		$center_lat = round((($kml2->maxLat + $kml2->minLat) / 2), 6);
		$center_lon = round((($kml2->maxLon + $kml2->minLon) / 2), 6);

		// 
		$data = array("self_path"            => $kml_filepath,
		              "self_url"             => $kml_url,
		              "gpx_path"             => $gpx_filepath,
		              "gpx_url"              => $gpx_url,
		              
		              "min_lat"              => $kml2->minLat,
		              "max_lat"              => $kml2->maxLat,
		              "min_lon"              => $kml2->minLon,
		              "max_lon"              => $kml2->maxLon,
					  
					  "center_lat"           => $center_lat,
					  "center_lon"           => $center_lon,
		              
		              "line_color"           => $gtogm_info->get_option("line_color"), 
		              "line_weight"          => $gtogm_info->get_option("line_weight"), 
		              "line_opacity"         => $gtogm_info->get_option("line_opacity"), 
		              "kashmir"              => $gtogm_info->get_option("kashmir"),
		              "google_maps_api_key"  => $gtogm_info->get_option("google_maps_api_key"),
		              "overview_map_control" => $gtogm_info->get_option("control"),
		              "map_type_control"     => $gtogm_info->get_option("map_type_control"),
		              "map_type"             => $gtogm_info->get_option("map_type"),
		              "map_width"            => $gtogm_info->get_option("map_width"),
		              "map_height"           => $gtogm_info->get_option("map_height"),
		              "open_google_maps"     => $gtogm_info->get_option("open_google_maps"),
					  "zoom_level"           => $gtogm_info->get_option("zoom_level")
		              );
		// 関連情報（アタッチメント）を登録する
		wp_update_attachment_metadata($kml_post_id, $data);
		
		
		return $file;
	}
	
	// 
	// GPXファイルのURLよりKMLファイルのポストのIDを取得する
	// 
	function get_kml_postid($gpx_url = "") {
		global $wpdb;
		
		if (preg_match("/\.[gG][pP][xX]$/", $gpx_url) == 1) {
			$query = "select ID from " . $wpdb->posts . " where guid = '" . preg_replace("/\.[gG][pP][xX]$/", ".kml", $gpx_url) . "' and post_parent = 0";
			
			$results = $wpdb->get_results($query);
			
			if ($results) {
				foreach ($results as $row) {
					return $row->ID;
				}
			}
		}
		return false;
		
	}

}


/*---------------------------------------------------------

    KMLファイル生成クラス

---------------------------------------------------------*/
class KMLFile {
	
	var $minLat = 90;
	var $maxLat = -90;
	var $minLon = 180;
	var $maxLon = -180;
	var $filename = "";
	
	//
	//  コンストラクタ
	//
	function __construct($gpx_path, $color, $weight, $alpha, $kashmir, $icon_url, $icon_width, $icon_height, $label_color) {
		
		// GPXファイル読み込み
		$xml = simplexml_load_file($gpx_path);
		if (!$xml) {
			return false;
		}
		
		$kml_head = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n";
		$kml_head .= "<kml xmlns=\"http://earth.google.com/kml/2.0\">\r\n";
		$kml_head .= "<Document>\r\n";
		
		// アイコンスタイル
		$kml_style .= "\t<Style id=\"istyle\">\r\n";
		$kml_style .= "\t\t<IconStyle>\r\n";
		$kml_style .= "\t\t\t<Icon>\r\n";
		$kml_style .= "\t\t\t\t<href>\r\n";
		$kml_style .= "\t\t\t\t\t" . $icon_url . "\r\n";
		$kml_style .= "\t\t\t\t</href>\r\n";
		$kml_style .= "\t\t\t</Icon>\r\n";
		$kml_style .= "\t\t\t<hotSpot x=\"0\" y=\"0\" xunits=\"fraction\" yunits=\"fraction\" />\r\n";
		$kml_style .= "\t\t</IconStyle>\r\n";
//		$kml_style .= "\t\t\t\t<LabelStyle>\r\n";
//		$kml_style .= "\t\t\t\t\t<color>" . $this->toABGR($label_color, sprintf("%02s", dechex(255))) . "</color>\r\n";
//		$kml_style .= "\t\t\t\t\t<colorMode>normal</colorMode>\r\n";
//		$kml_style .= "\t\t\t\t\t<scale>1.0</scale>\r\n";
//		$kml_style .= "\t\t\t\t</LabelStyle>\r\n";
		$kml_style .= "\t</Style>\r\n";
		
		
		$kml_data .= "\t<Folder>\r\n";
		$kml_data .= "\t\t<name>" . basename($gpx_path,".gpx") . "</name>\r\n";
		$kml_data .= "\t\t<description>" . $xml->xmlns . "</description>\r\n";
		
		
		$line_style_id = 1;
		$line_style = array();
		
		// <trk> トラックデータ
		foreach ($xml->trk as $trk) {
			$kml_data .= "\t\t<Placemark>\r\n";
			$kml_data .= "\t\t\t<name>" . (string) $trk->name . "</name>\r\n";
			$kml_data .= "\t\t\t<description></description>\r\n";
//			$kml_data .= "\t\t\t<Style>\r\n";
//			$kml_data .= "\t\t\t\t<LineStyle>\r\n";
			
			if ($kashmir == "1" && preg_match("/Kashmir3D/", $xml->attributes()->creator)) {
				// カシミール3D独自  カシミールもBGR？
				$hex_argb = sprintf("%02s", dechex($alpha)) . (string) $trk->extensions->children('kashmir3d', true)->line_color;
				$weight = $trk->extensions->children('kashmir3d', true)->line_size;
			} else {
				$hex_argb = $this->toABGR($color, sprintf("%02s", dechex($alpha)));
			}
			
			// ラインスタイル生成
			$current_line_style = array("color" => (string) $hex_argb, "width" => (string) $weight); // 型指定しておかないと比較でうまくいかない。
			$line_style_key = array_search($current_line_style, $line_style);
			if ($line_style_key === false) {
				// 同じスタイルが存在していない場合にスタイルを追加
				$line_style_name = "lstyle" . $line_style_id++;
				$kml_style .= "\t<Style id=\"" . $line_style_name . "\">\r\n";
				$kml_style .= "\t\t<LineStyle>\r\n";
				$kml_style .= "\t\t\t<color>" . $hex_argb . "</color>\r\n";
				$kml_style .= "\t\t\t<width>" . $weight . "</width>\r\n";
				$kml_style .= "\t\t</LineStyle>\r\n";
				$kml_style .= "\t</Style>\r\n";
				// これ以降の比較のためスタイル情報を保存
				$line_style[$line_style_name] = $current_line_style;
			} else {
				$line_style_name = $line_style_key;
			}
			
//			$kml_data .= "\t\t\t\t\t<color>" . $hex_argb . "</color>\r\n";
//			$kml_data .= "\t\t\t\t\t<width>" . $weight . "</width>\r\n";
//			$kml_data .= "\t\t\t\t</LineStyle>\r\n";
//			$kml_data .= "\t\t\t</Style>\r\n";
			$kml_data .= "\t\t\t<LineString>\r\n";
			$kml_data .= "\t\t\t\t<coordinates>\r\n";
			
			foreach ($trk->trkseg->trkpt as $trkpt) {
				$kml_data .= "\t\t\t\t\t" . (string) $trkpt["lon"] . "," . (string) $trkpt["lat"] . ",0\r\n";
				
				// 中心と範囲を割り出すため
				$lat = (string) $trkpt["lat"];
				$lon = (string) $trkpt["lon"];
				if ((float) $this->minLat > (float) $lat) {
					$this->minLat = $lat;
				}
				if ((float) $this->maxLat < (float) $lat) {
					$this->maxLat = $lat;
				}
				if ((float) $this->minLon > (float) $lon) {
					$this->minLon = $lon;
				}
				if ((float) $this->maxLon < (float) $lon) {
					$this->maxLon = $lon;
				}
			}
			
			$kml_data .= "\t\t\t\t</coordinates>\r\n";
			$kml_data .= "\t\t\t</LineString>\r\n";
			$kml_data .= "\t\t\t<styleUrl>#" . $line_style_name . "</styleUrl>\r\n";
			$kml_data .= "\t\t</Placemark>\r\n";
		}
		
		// <rte> ルートデータ
		$kml_data_route = "";
		foreach ($xml->rte as $rte) {
			
			// ルートのラインデータ
			$kml_data_route .= "\t\t<Placemark>\r\n";
			$kml_data_route .= "\t\t\t<name>" . (string) $rte->name . "</name>\r\n";
			$kml_data_route .= "\t\t\t<description></description>\r\n";
//			$kml_data_route .= "\t\t\t<Style>\r\n";
//			$kml_data_route .= "\t\t\t\t<LineStyle>\r\n";
			
			if ($kashmir == "1" && preg_match("/Kashmir3D/", $xml->attributes()->creator)) {
				// カシミール3D独自  カシミールもBGR？
				$hex_argb = sprintf("%02s", dechex($alpha)) . (string) $rte->extensions->children('kashmir3d', true)->line_color;
				$weight = $rte->extensions->children('kashmir3d', true)->line_size;
			} else {
				$hex_argb = $this->toABGR($color, sprintf("%02s", dechex($alpha)));
			}
			
			// ラインスタイル生成
			$current_line_style = array("color" => (string) $hex_argb, "width" => (string) $weight); // 型指定しておかないと比較でうまくいかない。
			$line_style_key = array_search($current_line_style, $line_style);
			if ($line_style_key === false) {
				// 同じスタイルが存在していない場合にスタイルを追加
				$line_style_name = "lstyle" . $line_style_id++;
				$kml_style .= "\t<Style id=\"" . $line_style_name . "\">\r\n";
				$kml_style .= "\t\t<LineStyle>\r\n";
				$kml_style .= "\t\t\t<color>" . $hex_argb . "</color>\r\n";
				$kml_style .= "\t\t\t<width>" . $weight . "</width>\r\n";
				$kml_style .= "\t\t</LineStyle>\r\n";
				$kml_style .= "\t</Style>\r\n";
				// これ以降の比較のためスタイル情報を保存
				$line_style[$line_style_name] = $current_line_style;
			} else {
				$line_style_name = $line_style_key;
			}
			
//			$kml_data_route .= "\t\t\t\t\t<color>" . $hex_argb . "</color>\r\n";
//			$kml_data_route .= "\t\t\t\t\t<width>" . $weight . "</width>\r\n";
//			$kml_data_route .= "\t\t\t\t</LineStyle>\r\n";
//			$kml_data_route .= "\t\t\t</Style>\r\n";
			$kml_data_route .= "\t\t\t<LineString>\r\n";
			$kml_data_route .= "\t\t\t\t<coordinates>\r\n";
			
			// ルートのポイントデータ
			foreach ($rte->rtept as $rtept) {
				$kml_data .= "\t\t<Placemark>\r\n";
				$kml_data .= "\t\t\t<name>" . $rtept->name . "</name>\r\n";
				$kml_data .= "\t\t\t<description></description>\r\n";
//				$kml_data .= "\t\t\t<Style>\r\n";
//				$kml_data .= "\t\t\t\t<IconStyle>\r\n";
//				$kml_data .= "\t\t\t\t\t<Icon>\r\n";
//				$kml_data .= "\t\t\t\t\t\t<href>" . $icon_url . "</href>\r\n";
//				$kml_data .= "\t\t\t\t\t\t<x>0</x>\r\n";
//				$kml_data .= "\t\t\t\t\t\t<y>0</y>\r\n";
//				$kml_data .= "\t\t\t\t\t\t<w>" . $icon_width . "</w>\r\n";
//				$kml_data .= "\t\t\t\t\t\t<h>" . $icon_height . "</h>\r\n";
//				$kml_data .= "\t\t\t\t\t</Icon>\r\n";
//				$kml_data .= "\t\t\t\t\t<scale>1.0</scale>\r\n";
//				$kml_data .= "\t\t\t\t</IconStyle>\r\n";
//				$kml_data .= "\t\t\t\t<LabelStyle>\r\n";
//				$kml_data .= "\t\t\t\t\t<color>" . $this->toABGR($label_color, sprintf("%02s", dechex(255))) . "</color>\r\n";
//				$kml_data .= "\t\t\t\t\t<colorMode>normal</colorMode>\r\n";
//				$kml_data .= "\t\t\t\t\t<scale>1.0</scale>\r\n";
//				$kml_data .= "\t\t\t\t</LabelStyle>\r\n";
//				$kml_data .= "\t\t\t</Style>\r\n";
				$kml_data .= "\t\t\t<Point>\r\n";
				$kml_data .= "\t\t\t\t<coordinates>\r\n";
				$kml_data .= "\t\t\t\t\t" . (string) $rtept["lon"] . "," . (string) $rtept["lat"] . ",0\r\n";
				$kml_data .= "\t\t\t\t</coordinates>\r\n";
				$kml_data .= "\t\t\t</Point>\r\n";
				$kml_data .= "\t\t\t<styleUrl>#istyle</styleUrl>\r\n";
				$kml_data .= "\t\t</Placemark>\r\n";
				
				// ルートのラインデータ
				$kml_data_route .= "\t\t\t\t\t" . (string) $rtept["lon"] . "," . (string) $rtept["lat"] . ",0\r\n";
				
				// 中心と範囲を割り出すため
				$lat = (string) $rtept["lat"];
				$lon = (string) $rtept["lon"];
				if ((float) $this->minLat > (float) $lat) {
					$this->minLat = $lat;
				}
				if ((float) $this->maxLat < (float) $lat) {
					$this->maxLat = $lat;
				}
				if ((float) $this->minLon > (float) $lon) {
					$this->minLon = $lon;
				}
				if ((float) $this->maxLon < (float) $lon) {
					$this->maxLon = $lon;
				}
			}
			
			$kml_data_route .= "\t\t\t\t</coordinates>\r\n";
			$kml_data_route .= "\t\t\t</LineString>\r\n";
			$kml_data_route .= "\t\t\t<styleUrl>#" . $line_style_name . "</styleUrl>\r\n";
			$kml_data_route .= "\t\t</Placemark>\r\n";
		}
		$kml_data .= $kml_data_route;
		
		// <wpt> ウェイポイントデータ
		foreach ($xml->wpt as $wpt) {
			$kml_data .= "\t\t<Placemark>\r\n";
			$kml_data .= "\t\t\t<name>" . $wpt->name . "</name>\r\n";
			$kml_data .= "\t\t\t<description></description>\r\n";
//			$kml_data .= "\t\t\t<Style>\r\n";
//			$kml_data .= "\t\t\t\t<IconStyle>\r\n";
//			$kml_data .= "\t\t\t\t\t<Icon>\r\n";
//			$kml_data .= "\t\t\t\t\t\t<href>" . $icon_url . "</href>\r\n";
//			$kml_data .= "\t\t\t\t\t\t<x>0</x>\r\n";
//			$kml_data .= "\t\t\t\t\t\t<y>0</y>\r\n";
//			$kml_data .= "\t\t\t\t\t\t<w>" . $icon_width . "</w>\r\n";
//			$kml_data .= "\t\t\t\t\t\t<h>" . $icon_height . "</h>\r\n";
//			$kml_data .= "\t\t\t\t\t</Icon>\r\n";
//			$kml_data .= "\t\t\t\t\t<scale>1.0</scale>\r\n";
//			$kml_data .= "\t\t\t\t</IconStyle>\r\n";
//			$kml_data .= "\t\t\t\t<LabelStyle>\r\n";
//			$kml_data .= "\t\t\t\t\t<color>" . $this->toABGR($label_color, sprintf("%02s", dechex(255))) . "</color>\r\n";
//			$kml_data .= "\t\t\t\t\t<colorMode>normal</colorMode>\r\n";
//			$kml_data .= "\t\t\t\t\t<scale>1.0</scale>\r\n";
//			$kml_data .= "\t\t\t\t</LabelStyle>\r\n";
//			$kml_data .= "\t\t\t</Style>\r\n";
			$kml_data .= "\t\t\t<Point>\r\n";
			$kml_data .= "\t\t\t\t<coordinates>\r\n";
			$kml_data .= "\t\t\t\t\t" . (string) $wpt["lon"] . "," . (string) $wpt["lat"] . ",0\r\n";
			$kml_data .= "\t\t\t\t</coordinates>\r\n";
			$kml_data .= "\t\t\t</Point>\r\n";
			$kml_data .= "\t\t\t<styleUrl>#istyle</styleUrl>\r\n";
			$kml_data .= "\t\t</Placemark>\r\n";
			
			// 中心と範囲を割り出すため
			$lat = (string) $wpt["lat"];
			$lon = (string) $wpt["lon"];
			if ((float) $this->minLat > (float) $lat) {
				$this->minLat = $lat;
			}
			if ((float) $this->maxLat < (float) $lat) {
				$this->maxLat = $lat;
			}
			if ((float) $this->minLon > (float) $lon) {
				$this->minLon = $lon;
			}
			if ((float) $this->maxLon < (float) $lon) {
				$this->maxLon = $lon;
			}
		}
				
		
		$kml_data .= "\t</Folder>\r\n";
		
		$kml_foot .= "</Document>\r\n";
		$kml_foot .= "</kml>\r\n";
		
		$kml_text = $kml_head . $kml_style . $kml_data . $kml_foot;
		
		$kml_file = preg_replace("/.[gG][pP][xX]$/", ".kml", $gpx_path);
		$fp = fopen($kml_file, "w");
		fwrite($fp, mb_convert_encoding($kml_text, "UTF-8"));
		fclose($fp);
		
		$this->filename = basename($kml_file);
	}
	
	//
	//  8進数のRGBをABGRに変換する。（Google Maps の色は通常とは逆順のBGRでその頭に透過が付いたABGRとなる）
	//
	function toABGR($hex_rgb, $hex_alpha) {
		$hex_red = substr($hex_rgb, 0, 2);
		$hex_green = substr($hex_rgb, 2, 2);
		$hex_blue = substr($hex_rgb, 4, 2);
		
		return $hex_alpha . $hex_blue . $hex_green . $hex_red;
	}
	
}


?>