<?php

/*

メディアの追加画面でも、ショートコード内に入れる情報が、設定できると便利だね。

*/

class GTOGM_ShortCode {


	//
	//  コンストラクタ
	//
	function __construct() {

		add_action("wp_print_scripts", array(&$this, "gmap_print_scripts"));
		//add_action("wp_head", "gmap_print_scripts");
		
		// ショートコード
		add_shortcode('gmap', array(&$this, 'gmap_shortcode_handler'));
	}

	function gmap_print_scripts() {
		
		global $post;
		
		if (is_single() || is_page()) {
			// 記事内のショートコードの有無を判定
			if (preg_match("/\[gmap [^\]]+\]/", $post->post_content)) {
				// プラグイン情報管理インスタンス生成
				$gtogm_info = new GTOGM_Info();
				echo '<script src="http://maps.google.com/maps?file=api&v=2.x&key=' . $gtogm_info->get_option("google_maps_api_key") . '" type="text/javascript"></script>' . "\n";
			}
		}
	}



	// [gmap id="xx"]
	function gmap_shortcode_handler($atts) {
		
		if (!is_single() && !is_page()) {
			return "";
		}
		
		if (!isset($atts["id"]) 
			|| !isset($atts["width"]) 
			|| !isset($atts["height"]) 
			|| !isset($atts["clat"]) 
			|| !isset($atts["clon"])
			|| !isset($atts["zoom"])
			|| !isset($atts["type"])
			|| !isset($atts["control"])
			|| !isset($atts["mtcontrol"])
			|| !isset($atts["glink"])
			) {
			return "";
		}
		
		if ((1 != preg_match("/^[1-9][0-9]*$/", $atts["id"])) 
			|| (1 != preg_match("/^[1-9][0-9]*$/", $atts["width"]))
			|| (1 != preg_match("/^[1-9][0-9]*$/", $atts["height"]))
			|| (1 != preg_match("/^-?([1-9][0-9]?)?[0-9](\.[0-9]+)?$/", $atts["clat"]))
			|| (1 != preg_match("/^-?([1-9][0-9]?)?[0-9](\.[0-9]+)?$/", $atts["clon"]))
			|| (1 != preg_match("/^(-1|([1-9][0-9]*)?[0-9])$/", $atts["zoom"]))
			|| (1 != preg_match("/^(1|2|4|8|16|32|64|128)$/", $atts["type"]))
			|| (1 != preg_match("/^([1-9][0-9]*)?[0-9]$/", $atts["control"]))
			|| (1 != preg_match("/^([1-9][0-9]*)?[0-9]$/", $atts["mtcontrol"]))
			|| (1 != preg_match("/^([1-9][0-9]*)?[0-9]$/", $atts["glink"]))
			) {
			return "";
		}
		
		// [gmap id="293" width="480" height="480" clat="36.562393" clon="137.630271" zoom="-1" type="1" control="15" mtcontrol="31" glink="1"]
		
		$kml_id = $atts["id"];
		
		// KMLファイルのアタッチメント情報を取得
		$kml_metadata = wp_get_attachment_metadata($kml_id);
		
		// KMLファイルのポストを取得
	//	$kml_post = get_post($kml_id);
		
		
		$kml_url = $kml_metadata["self_url"];
		
		// 
		$gmap2 = new GTOGM_GMap2($kml_url, "gmapid" . $kml_id);
		
		
		$gmap2->map_type_control     = (int) $atts["mtcontrol"];
		$gmap2->control              = (int) $atts["control"];
		$gmap2->map_type             = (int) $atts["type"];
		$gmap2->zoom_level           = (int) $atts["zoom"];
		
		$gmap2->center_lat           = $atts["clat"];
		$gmap2->center_lon           = $atts["clon"];
		
		$map_width                   = $kml_metadata["map_width"];
		$map_height                  = $kml_metadata["map_height"];
		$open_google_maps            = $kml_metadata["open_google_maps"];
		$download_kml_file           = $kml_metadata["download_kml_file"];

		$open_google_maps_html = "";
		if ($atts["glink"] == 1) {
			// GoogleMapsパラメータ用の地図タイプをセット
			if ($gmap2->map_type & GTOGM_Info::G_NORMAL_MAP) {
				$t = "m";
			} else if ($gmap2->map_type & GTOGM_Info::G_SATELLITE_MAP) {
				$t = "k";
			} else if ($gmap2->map_type & GTOGM_Info::G_HYBRID_MAP) {
				$t = "h";
			} else if ($gmap2->map_type & GTOGM_Info::G_PHYSICAL_MAP) {
				$t = "p";
			} else if ($gmap2->map_type & GTOGM_Info::G_SATELLITE_3D_MAP) {
				$t = "e";
			} else {
				$t = "m";
			}
			$open_google_maps_html = "<div id=\"linkOpen\"><a href=\"http://www.google.co.jp/maps?q=" . $kml_url . "&t=$t\" target=\"_blank\">" . __("Open Google Maps", "gtogm") . "</a></div>";
		}
		
		
		$gmap2->post_meta_min_lat = $kml_metadata["min_lat"];
		$gmap2->post_meta_max_lat = $kml_metadata["max_lat"];
		$gmap2->post_meta_min_lon = $kml_metadata["min_lon"];
		$gmap2->post_meta_max_lon = $kml_metadata["max_lon"];
		
		$gmap2->map_width  = $kml_metadata["map_width"];
		$gmap2->map_height = $kml_metadata["map_height"];
		
		return $gmap2->get_map_html() . $open_google_maps_html;
	}
}

/*---------------------------------------------------------

    GMap2描画用javascript生成クラス

---------------------------------------------------------*/
class GTOGM_GMap2 {
	
	public $map_type_control;
	public $control;
	public $map_type;
	
	public $post_meta_min_lat;
	public $post_meta_max_lat;
	public $post_meta_min_lon;
	public $post_meta_max_lon;
	
	public $map_width;
	public $map_height;
	
	public $center_lat;
	public $center_lon;
	
	public $zoom_level;
	
	private $kml_file_url;
	private $anchor_href;
	
	private $map_id;
	
	//
	//  コンストラクタ
	//
	function __construct($kml_url, $map_id) {
		
		$this->kml_file_url = $kml_url;
		$this->anchor_href = $anchor_href;
		
		$this->map_id = $map_id;
		
		$this->map_type_control = 0;
		$this->control = 0;
		$this->map_type = 1;
		$this->zoom_level = -1;
		
		$this->post_meta_min_lat = "0";
		$this->post_meta_max_lat = "0";
		$this->post_meta_min_lon = "0";
		$this->post_meta_max_lon = "0";
		
	}
	
	//
	// GMap2出力HTML取得
	//
	function get_map_html() {
		
		$map_type = 1;
		if ($this->map_type & GTOGM_Info::G_NORMAL_MAP) {
			$map_type = "G_NORMAL_MAP";
		} else if ($this->map_type & GTOGM_Info::G_HYBRID_MAP) {
			$map_type = "G_HYBRID_MAP";
		} else if ($this->map_type & GTOGM_Info::G_SATELLITE_MAP) {
			$map_type = "G_SATELLITE_MAP";
		} else if ($this->map_type & GTOGM_Info::G_PHYSICAL_MAP) {
			$map_type = "G_PHYSICAL_MAP";
		} else if ($this->map_type & GTOGM_Info::G_SATELLITE_3D_MAP) {
			$map_type = "G_SATELLITE_3D_MAP";
		}
		
		$map_type_control_js = "";
		if ($this->map_type_control & GTOGM_Info::G_NORMAL_MAP) {
			$map_type_control_js .= "$this->map_id.addMapType(G_NORMAL_MAP);" . "\n";
		}
		if ($this->map_type_control & GTOGM_Info::G_HYBRID_MAP) {
			$map_type_control_js .= "$this->map_id.addMapType(G_HYBRID_MAP);" . "\n";
		}
		if ($this->map_type_control & GTOGM_Info::G_SATELLITE_MAP) {
			$map_type_control_js .= "$this->map_id.addMapType(G_SATELLITE_MAP);" . "\n";
		}
		if ($this->map_type_control & GTOGM_Info::G_PHYSICAL_MAP) {
			$map_type_control_js .= "$this->map_id.addMapType(G_PHYSICAL_MAP);" . "\n";
		}
		if ($this->map_type_control & GTOGM_Info::G_SATELLITE_3D_MAP) {
			$map_type_control_js .= "$this->map_id.addMapType(G_SATELLITE_3D_MAP);" . "\n";
		}
		
		$control_js = "";
		if ($this->control & GTOGM_Info::LARGE_MAP_CONTROL) {
			// 左側のコントロールボタンを表示
			$control_js .= "$this->map_id.addControl(new GLargeMapControl());" . "\n";
		}
		if ($this->control & GTOGM_Info::MAP_TYPE_CONTROL) {
			// 地図タイプ切り替えボタンを表示
			$control_js .= "$this->map_id.addControl(new GMapTypeControl());" . "\n";
		}
		if ($this->control & GTOGM_Info::SCALE_CONTROL) {
			// 左下のスケール表示
			$control_js .= "$this->map_id.addControl(new GScaleControl());" . "\n";
		}
		if ($this->control & GTOGM_Info::OVERVIEW_MAP_CONTROL) {
			// 右下の地図位置表示
			$control_js .= "$this->map_id.addControl(new GOverviewMapControl());" . "\n";
		}
		
		$map_user_control_js .= "$this->map_id.enableDragging();" . "\n";
		$map_user_control_js .= "$this->map_id.enableDoubleClickZoom();" . "\n";
		$map_user_control_js .= "$this->map_id.enableContinuousZoom();" . "\n";
		$map_user_control_js .= "$this->map_id.disableGoogleBar();" . "\n";
		$map_user_control_js .= "$this->map_id.enableScrollWheelZoom();" . "\n";
		
		$zoom_level_js = "";
		if ($this->zoom_level >= 0) {
			$zoom_level_js .= 'var zoomlevel = ' . $this->zoom_level . ";\n";
		} else {
			$zoom_level_js .= 'var rectObj = new GLatLngBounds(new GLatLng(' . $this->post_meta_min_lat . ', ' . $this->post_meta_min_lon . '), new GLatLng(' . $this->post_meta_max_lat . ', ' . $this->post_meta_max_lon . "));\n";
			$zoom_level_js .= 'var zoomlevel = ' . $this->map_id . '.getBoundsZoomLevel(rectObj);' . "\n";
		}
		
		$html = <<< EOF
<div id="{$this->map_id}" style="width:{$this->map_width}px;height:{$this->map_height}px;"></div>
<script type="text/javascript">
function {$this->map_id}() {
{$this->map_id} = new GMap2(document.getElementById("{$this->map_id}"));
// 標準の表示を
{$this->map_id}.setMapType({$map_type});
{$map_user_control_js}
{$map_type_control_js}
{$control_js}
{$zoom_level_js}
{$this->map_id}.setCenter(new GLatLng({$this->center_lat}, {$this->center_lon}), zoomlevel);
var gx = new GGeoXml("{$this->kml_file_url}");
{$this->map_id}.addOverlay(gx);
}



{$this->map_id}();
</script>
EOF;

		return $html;
	}
}

?>